<?php

namespace core\container;
use core\{database\Database, utility\Utility, theme\Theme, user\User, route\Route, translation\Translation as T};

class Container {
	use Api, Form;

	function admin() {
		$containers = Database::select('container')
			->where('status', 1)
			->sort('heft', 'ASC')
			->fetch('assoc_all');

		$data = [
			'containers' => $containers,
			'container_list' => T::get('Список контейнеров'),
			'container_add' => T::get('Добавить контейнер'),
			'name' => T::get('Название'),
			'entity' => T::get('Сущность'),
			'title' => T::get('Заголовок'),
			'actions' => T::get('Действия'),
			'change' => T::get('изменить'),
			'delete' => T::get('удалить'),
			'containers_no' => T::get('Контейнеры отсутствуют'),
			
			'add_route' => Route::render_route('admin/container/add'),
			'edit_route' => Route::render_route('admin/container/edit'),
			'delete_route' => Route::render_route('admin/container/delete'),
		];

		$result = (new Theme)->get_template(__DIR__, 'admin', $data);

		return $result;
	}

	function show() {
		$theme = new Theme();
		$user = new User();

		$uri = Route::get_route(false, null, false);

		$theme_object = $theme->get_list(true);
		$regions_names = $theme_object ? $theme_object->regions : [];
		$regions_names = $regions_names ? unserialize($regions_names) : [];

		$entities = Route::get_entities();

		$regions = [];
		foreach ($regions_names as $region_name) {
			$r_container = Database::select('container')
				->where('region', $region_name)
				->where('status', 1)
				->sort('heft', 'ASC')
				->fetch('assoc_all');

			$region_value = '';
			foreach ($r_container as $row) {
				if ($row['entity']) {
					$entity = $entities[$row['entity']];
					new $entity;
				}

				$access = $row['access'] ? unserialize($row['access']) : [];
				if (!$user->get_access($access, '', true)) {
					continue;
				}

				$region = $row['body'];
				if ($row['display'] == 'uri' || $row['display'] == 'uri_no') {
					$pattern = '/^'.str_replace('*', '[0-9a-z\.\/\-]+', addcslashes($row['display_value'], '/')).'$/';

					if ($row['display'] == 'uri') {
						$region = preg_match($pattern, $uri) ? $row['body'] : '';
					} else {
						$region = !preg_match($pattern, $uri) ? $row['body'] : '';
					}
				} else if ($row['display'] == 'content_type') {
					$content_type_id = Database::select('content_types')
						->data(['id'])
						->where('name', $row['display_value'])
						->fetch('field');

					$route_alias = Database::select('route_aliases')
						->data(['entity_id'])
						->where('alias', $uri)
						->where('type', 'content')
						->fetch('field');

					$r_content = Database::select('content')
						->where('id', $route_alias)
						->where('type', $content_type_id)
						->fetch('assoc');

					$region = $r_content ? $row['body'] : '';
				}

				if (mb_substr($region, 0, 5) == '<?php') {
					ob_start();
					@eval(mb_substr($region, 6));
					$region = ob_get_clean();
				}
				$region = $region ? ($row['title'] ? '<h4>'.$row['title'].'</h4>'.$region : $region) : '';

				$region_value .= $region;
			}

			$regions[$region_name] = $region_value;
		}

		$body_width = 'hight';
		if (!empty($regions['left']) && !empty($regions['right'])) {
			$body_width = 'low';
		} else if (!empty($regions['left']) || !empty($regions['right'])) {
			$body_width = 'medium';
		}

		$regions_list = ['left', 'right'];
		foreach ($regions_list as $region) {
			if (!empty($regions[$region])) {
				$region_html = $regions[$region];

				$data = compact('region', 'region_html');
				$regions[$region] = PHP_EOL.$theme->get_template(__DIR__, 'show', $data);
			}
		}

		$regions['body_width'] = $body_width;

		return $regions;
	}

	function add() {
		$user = new User;

		if (!$_POST) {
			$theme = (new Theme)->get_theme();
			$regions_data = unserialize($theme->regions);

			$regions = [
				'' => T::get('Не указано'),
			];

			foreach ($regions_data as $key => $value) {
				$regions[$value] = $value;
			}

			$access = [];
			foreach ($_POST as $key => $value) {
				if (mb_strpos($key, 'access_') !== false) {
					$access[] = $value;
				}
			}
			$access = $user->get_roles_names($access);

			$data = [
				'name' => $_POST['name'] ?? '',
				'title' => $_POST['title'] ?? '',
				'body' => $_POST['body'] ?? '',
				'region' => $_POST['region'] ?? '',
				'display' => $_POST['display'] ?? '',
				'display_value' => $_POST['display_value'] ?? '',
				'heft' => $_POST['heft'] ?? '',
				'regions' => $regions,
				'access' => $access,
				'entity' => 0,
			];

			$result = $this->get_form($data);

			return $result;
		} else {
			$display = $_POST['display'] ?? '';

			$access = [];
			foreach ($_POST as $key => $value) {
				if (mb_strpos($key, 'access_') !== false) {
					$access[] = $value;
				}
			}
			$access = $user->get_roles_ids($access);
			$access = serialize($access);

			Database::insert('container')
				->data([
					'name' => $_POST['name'],
					'title' => $_POST['title'],
					'body' => $_POST['body'],
					'region' => $_POST['region'],
					'display' => $display,
					'display_value' => $_POST['display_value'],
					'heft' => $_POST['heft'],
					'access' => $access,
				])
				->execute();

			Utility::set_message('success');
			Route::go_to('admin/container');
		}
	}

	function edit() {
		$user = new User;

		$id = Route::get_route_id();
		$container = Database::select('container')
			->where('id', $id)
			->where('status', 1)
			->fetch('assoc');

		$result = '';
		if ($container) {
			if (!$_POST) {
				$theme = (new Theme)->get_theme();
				$regions_data = unserialize($theme->regions);
				$access_data = $container['access'] ? unserialize($container['access']) : [];
				$access_data = $user->get_roles_names($access_data);

				$regions = [
					'' => T::get('Не указано'),
				];

				foreach ($regions_data as $key => $value) {
					$regions[$value] = $value;
				}

				$access = [];
				foreach ($access_data as $key => $value) {
					$access[$value] = $value;
				}

				$data = [
					'name' => $_POST['name'] ?? $container['name'],
					'title' => $_POST['title'] ?? $container['title'],
					'body' => $_POST['body'] ?? $container['body'],
					'region' => $_POST['region'] ?? $container['region'],
					'display' => $_POST['display'] ?? $container['display'],
					'display_value' => $_POST['display_value'] ?? $container['display_value'],
					'heft' => $_POST['heft'] ?? $container['heft'],
					'regions' => $regions,
					'access' => $access,
					'entity' => $container['entity'],
				];

				$result = $this->get_form($data, 'edit');

				return $result;
			} else {
				$display = $_POST['display'] ?? '';

				$name = $container['entity'] ? $container['name'] : ($_POST['name'] ?? $container['name']);
				//$body = $container['entity'] ? $container['body'] : ($_POST['body'] ?? $container['body']);
				$body = $_POST['body'] ?? $container['body'];

				$access = [];
				foreach ($_POST as $key => $value) {
					if (mb_strpos($key, 'access_') !== false) {
						$access[] = $value;
					}
				}
				$access = $user->get_roles_ids($access);
				$access = serialize($access);

				Database::update('container')
					->data([
						'name' => $name,
						'title' => $_POST['title'],
						'body' => $body,
						'region' => $_POST['region'],
						'display' => $display,
						'display_value' => $_POST['display_value'],
						'heft' => $_POST['heft'],
						'access' => $access,
					])
					->where('id', $container['id'])
					->execute();

				Utility::set_message('success');
				Route::go_to('admin/container');
			}
		}

		return $result;
	}

	function delete() {
		$id = Route::get_route(true, 3);

		$r_container = Database::select('container')
			->where('id', $id)
			->where('status', 1)
			->fetch('assoc');

		$result = '';
		if ($r_container) {
			if (!$_POST) {
				$result = Utility::get_form_delete();
			} else if (isset($_POST['cancel'])) {
				Route::go_to('admin/container');
			} else {
				Database::delete('container')
					->where('id', $id)
					->execute();

				Utility::set_message('success');
				Route::go_to('admin/container');
			}
		}

		return $result;
	}
}
