<?php

namespace core\theme;
use core\{application\Application, utility\Utility, database\Database, route\Route, translation\Translation as T};

class Theme {
	use Api;

	function show() {
		$buffer = Utility::get_buffer();
		$body = $buffer['all'];

		$title = Application::get_title();
		$home_route = Route::render_route('');
		$title_page = $title;
		$name = Application::get_name();

		$title = $title ? $title.' - '.$name : $name;
		$language = T::get_language();

		$meta = '';
		$copyright = date('Y');

		$path_uri = Route::get_path(__DIR__);
		Utility::add_library('others/fonts/opensans/style.css', -1);
		Utility::add_library($path_uri.'/css/base.css');
		Utility::add_library($path_uri.'/css/theme.css');
		Utility::add_library('others/libraries/jquery/jquery.js', -1);

		extract($this->get_libraries());
		$status = $this->get_messages();

		$data = compact('status', 'body', 'copyright', 'title', 'title_page', 'home_route');
		$body = self::get_template(__DIR__, 'body', $data);
		
		$favicon_route = Route::render_route('favicon.ico');

		$data = compact('language', 'meta', 'title', 'css', 'body', 'js', 'favicon_route');
		$result = self::get_template(__DIR__, 'html', $data);

		return $result;
	}

	function admin() {
		$themes = array_merge(glob(EQSASH_ROOT.'/themes/*/*.ini'), glob(EQSASH_ROOT.'/custom/themes/*/*.ini'));

		if (!isset($_POST['save_config'])) {
			Application::set_title('Темы');

			$result_theme = Database::select('theme')
				->data(['name'])
				->fetch('assoc_all');

			foreach ($result_theme as $themes_names) {
				$check_theme = array_merge(glob(EQSASH_ROOT.'/themes/'.$themes_names['name']), glob(EQSASH_ROOT.'/custom/themes/'.$themes_names['name']));

				if (!$check_theme) {
					Database::delete('theme')
						->where('name', $themes_names['name'])
						->execute();
				}
			}

			$data = [
				'on' => T::get('Включено'),
				'name_phrase' => T::get('Название'),
				'version' => T::get('Версия'),
				'description' => T::get('Описание'),
				'screenshot' => T::get('Скриншот'),
				'save' => T::get('Подтвердить'),
			];

			foreach ($themes as $file) {
				$type = mb_strpos($file, 'custom') === false ? 'base' : 'custom';

				$ini_array = parse_ini_file($file);

				$theme_status = Database::select('theme')
					->data(['status'])
					->where('name',  $ini_array['name'])
					->fetch('field');

				$checked = ' ';
				if ($theme_status) {
					$checked = $theme_status == 1 ? ' checked ' : ' ';
				}

				$screenshot_path = $type == 'base' ? 'themes' : 'custom/themes';
				$screenshot_path .= '/'.$ini_array['name'].'/'.$ini_array['name'].'.png';

				$data['items'][] = [
					'name' => $ini_array['name'],
					'version' => $ini_array['version'],
					'description' => $ini_array['description'],
					'checked' => $checked,
					'type' => $type,
					'screenshot_path' => Route::render_route($screenshot_path),
				];
			}

			$result = self::get_template(__DIR__, 'admin', $data);

			return $result;
		} else {
			$this->save();
		}
	}

	function save() {
		unset($_POST['save_config']);

		$types = [];
		foreach ($_POST as $key => $value) {
			if (mb_strpos($key, '_type') !== false) {
				$name = mb_strcut($key, 0, -5);
				unset($_POST[$key]);
				$types[$name] = $value;
			}
		}

		$theme_name = Database::select('theme')
			->data(['name'])
			->where('status',  1)
			->fetch('field');

		if ($theme_name) {
			$array = $_POST;

			if (!array_search($theme_name, $array)) {
				Database::update('theme')
					->data(['status' => 0])
					->where('name',  $theme_name)
					->execute();
			}
		}

		if (!empty($_POST)) {
			$array = $_POST;

			foreach ($array as $name) {
				$theme_name = Database::select('theme')
					->data(['name'])
					->where('name',  $name)
					->fetch('field');

				if ($theme_name) {
					Database::update('theme')
						->data(['status' => 1])
						->where('name',  $theme_name)
						->execute();
				} else {
					$type = $types[$name];
					$path = $type == 'base' ? '' : '/custom';
					$path = EQSASH_ROOT.$path.'/themes/'.$name.'/'.$name.'.ini';

					$ini = parse_ini_file($path);

					$regions = explode(', ', $ini['regions']);
					$regions = serialize($regions);

					Database::insert('theme')
						->data([
							'name' => $name,
							'type' => $type,
							'regions' => $regions,
							'status' => 1,
						])
						->execute();
				}
			}
		}

		Utility::set_message('Настройки сохранены', 'green');
		unset($_POST);

		Utility::clear_cache();
		Route::go_to('admin/theme');
	}

	//

	function get_libraries() {
		$libraries = $GLOBALS['eqsash']['libraries'] ?? [];
		$cache = Utility::check_cache();
		$cache_path = 'others/data/cache';
		$cache_path_random = EQSASH_ROOT.'/'.$cache_path.'/random.txt';

		$map = [
			'css' => 'styles',
			'js' => 'scripts',
		];

		foreach ($map as $key => $value) {
			$libraries_section = $libraries[$key] ?? [];

			$cache_path_item = $cache_path.'/'.$value.'.'.$key;
			$cache_path_full_item = EQSASH_ROOT.'/'.$cache_path_item;

			if ($cache) {
				$cache_content = '';
				foreach ($libraries_section as $key_two => $value_two) {
					if ($value_two['local']) {
						$item_path = EQSASH_ROOT.'/'.$key_two;
						$cache_content .= file_exists($item_path) ? file_get_contents($item_path) : $key_two;
					} else {
						$cache_content .= file_get_contents($key_two);
					}
					$cache_content .= PHP_EOL.PHP_EOL;
				}

				if (!file_exists($cache_path_full_item) || file_get_contents($cache_path_full_item) != $cache_content) {
					$random = mt_rand();

					file_put_contents($cache_path_random, $random);
					file_put_contents($cache_path_full_item, $cache_content);
				} else {
					$random = file_get_contents($cache_path_random);
				}

				$cache_path_item = Route::render_route($cache_path_item.'?'.$random);
				${$key} = $key == 'css' ? '<link rel="stylesheet" href="'.$cache_path_item.'">' : '<script src="'.$cache_path_item.'"></script>';
				${$key} .= PHP_EOL;
			} else {
				${$key} = '';
				foreach ($libraries_section as $key_two => $value_two) {
					${$key} .= '	'.$value_two['code'].PHP_EOL;
				}
				${$key} = ltrim(${$key});
			}
		}

		$libraries = compact('css', 'js');

		return $libraries;
	}

	function get_messages() {
		$messages = '';
		if (isset($_SESSION['red_message'])) {
			$messages .= '<div class="message message-red">'.$_SESSION['red_message'].'</div>';
			unset($_SESSION['red_message']);
		}

		if (isset($_SESSION['green_message'])) {
			$messages .= '<div class="message message-green">'.$_SESSION['green_message'].'</div>';
			unset($_SESSION['green_message']);
		}

		return $messages;
	}

	function get_template($path, $name, $data = []) {
		$entity = Route::get_path($path, false, true, -1);
		$path_theme = Database::check() ? $this->get_theme('path').'/templates/'.$entity.'/'.$name.'.tpl' : '';
		$path_default = $path.'/templates/'.$name.'.tpl';

		$path = file_exists($path_theme) ? $path_theme : $path_default;
		$template = Utility::build_template($path, $data);

		return $template;
	}
}
