<?php

namespace core\utility\crypt;
use core\{utility\Utility};

class Crypt {
	public $algorithm;
	public $algorithm_full;	
    public $algorithm_hash = 'sha256';
	
    public $keys = [];
	
    function __construct($algorithm = 'AES') {
		$this->algorithm = $algorithm;
		$this->algorithm_full = $algorithm == 'AES' ? 'aes-256-ctr' : 'RSA';
	}
	
    function set_keys($keys) {
		$this->keys = $keys;
	}
	
    function get_keys() {
		return $this->keys;
	}
	
    function encrypt($data) {
		return $this->algorithm == 'AES' ? $this->encrypt_aes($data) : $this->encrypt_rsa($data);
	}
	
    function decrypt($data) {
		return $this->algorithm == 'AES' ? $this->decrypt_aes($data) : $this->decrypt_rsa($data);
	}
	
    function encrypt_aes($data) {
		$key_private = $this->keys[0];
		
        list($key_encryption, $key_auth) = $this->split_keys($this->base64_decode($key_private));
		
        $nonce = openssl_random_pseudo_bytes(openssl_cipher_iv_length($this->algorithm_full));
        $data_chipher = $nonce.openssl_encrypt($data, $this->algorithm_full, $key_encryption, OPENSSL_RAW_DATA, $nonce);

        $mac = hash_hmac($this->algorithm_hash, $data_chipher, $key_auth, true);
		$result = $this->base64_encode($mac.$data_chipher);
       
        return $result;
    }
  
    function decrypt_aes($data) {
		$key_private = $this->keys[0];
		
        $data = $this->base64_decode($data);
        list($key_encryption, $key_auth) = $this->split_keys($this->base64_decode($key_private));
		
		$result = '';
		if ($data) {
			$digest_length = $this->get_digest_length();
			$mac = mb_strcut($data, 0, $digest_length, '8bit');

			$data_chipher = mb_strcut($data, $digest_length, null, '8bit');
			$mac_calculate = hash_hmac($this->algorithm_hash, $data_chipher, $key_auth, true);

			if ($this->check_hash($mac, $mac_calculate)) {
				$random_number = openssl_cipher_iv_length($this->algorithm_full);
				$nonce = mb_strcut($data_chipher, 0, $random_number, '8bit');
				$data_chipher = mb_strcut($data_chipher, $random_number, null, '8bit');
				
				$result = openssl_decrypt($data_chipher, $this->algorithm_full, $key_encryption, OPENSSL_RAW_DATA, $nonce);
			}
		}

        return $result;
    }
	
	//
	
	function encrypt_rsa($data) {
		$key_public = $this->keys[1];
		
		$key_public_original = $key_public;
		
		if (mb_substr($key_public, 0, 10) != '-----BEGIN') {
			$key_public = self::base64_set_padding(strtr($key_public, '-_', '+/'));		
			$key_public = "-----BEGIN PUBLIC KEY-----\r\n".chunk_split($key_public, 64)."-----END PUBLIC KEY-----";
		}
		
		list($key_encryption, $key_auth) = $this->split_keys($this->base64_decode($key_public_original));		
        $nonce = openssl_random_pseudo_bytes(16);
		
		openssl_public_encrypt($data, $result, openssl_get_publickey($key_public));
		
        $data_chipher = $nonce.$result;

        $mac = hash_hmac($this->algorithm_hash, $data_chipher, $key_auth, true);
		$result = $this->base64_encode($mac.$data_chipher);
		
		return $result;
	}
	
	function decrypt_rsa($data) {
		$key_private = $this->keys[0];
		$key_public = $this->keys[1];
		
		$key_public_original = $key_public;
		
		if (mb_substr($key_private, 0, 10) != '-----BEGIN') {
			$key_private = self::base64_set_padding(strtr($key_private, '-_', '+/'));		
			$key_private = "-----BEGIN RSA PRIVATE KEY-----\r\n".chunk_split($key_private, 64)."-----END RSA PRIVATE KEY-----";
		}
		if (mb_substr($key_public, 0, 10) != '-----BEGIN') {
			$key_public = self::base64_set_padding(strtr($key_public, '-_', '+/'));		
			$key_public = "-----BEGIN PUBLIC KEY-----\r\n".chunk_split($key_public, 64)."-----END PUBLIC KEY-----";
		}

		$data = $this->base64_decode($data);
		
        list($key_encryption, $key_auth) = $this->split_keys($this->base64_decode($key_public_original));
		
		$result = '';
		if ($data) {
			$digest_length = $this->get_digest_length();
			$mac = mb_strcut($data, 0, $digest_length, '8bit');

			$data_chipher = mb_strcut($data, $digest_length, null, '8bit');
			$mac_calculate = hash_hmac($this->algorithm_hash, $data_chipher, $key_auth, true);

			if ($this->check_hash($mac, $mac_calculate)) {
				$data = mb_strcut($data, $this->get_digest_length(), null, '8bit');
				$data = mb_strcut($data, 16, null, '8bit');

				openssl_private_decrypt($data, $result, $key_private);
			}
		}
		
		return $result;
	}
	
	//
	
    protected function split_keys($key) {
		$result = [
			hash_hmac($this->algorithm_hash, 'ENCRYPTION', $key, true),
            hash_hmac($this->algorithm_hash, 'AUTHENTICATION', $key, true)
        ];
		
		return $result;
    }
	
    function get_digest_length() {
		$digest_length = mb_strlen(hash($this->algorithm_hash, '', true), '8bit');
		
		return $digest_length;
	}
	
    protected function check_hash($a, $b) {	
        if (function_exists('hash_equals')) {
            $result = hash_equals($a, $b);
        } else {		
			$nonce = openssl_random_pseudo_bytes(32);		
			$result = hash_hmac($this->algorithm_hash, $a, $nonce) === hash_hmac($this->algorithm_hash, $b, $nonce);
		}
		
        return $result;
    }
	
	function base64_encode($data) {
		return rtrim(strtr(base64_encode($data), '+/', '-_'), '=');
	}

	function base64_decode($data) {
		$data = strtr($data, '-_', '+/');		
		$result = base64_decode(self::base64_set_padding($data));
		
		return $result;
	}

	static function base64_set_padding($data) {
		$data_length = mb_strlen($data);
		
		$count = $data_length;
		for ($count; $count <= $data_length + 4; $count++) {
			if ($count % 4 == 0) {
				break;
			}
		}
		
		$result = str_pad($data, $count, '=');
		
		return $result;
	}
	
	//
	
    function generate_rsa_keys() {
		$crypt = Utility::get_data('crypt');
		
		if (!$crypt) {
			$key = openssl_pkey_new([
				'digest_alg' => 'sha256',
				'private_key_bits' => 2048,
				'private_key_type' => OPENSSL_KEYTYPE_RSA,
			]);
			
			openssl_pkey_export($key, $rsa_private);		
			$rsa_public = openssl_pkey_get_details($key)['key'] ?? '';
			
			$crypt['rsa_private'] = $rsa_private;
			$crypt['rsa_public'] = $rsa_public;
			
			Utility::set_data('crypt', $crypt);
		}
		
		return $crypt;
	}
	
	function convert_pem_der($key) {
		$key = trim(strtr($key, '+/', '-_'), '=');
		$key = str_replace("\n", '', str_replace("\r\n", '', $key));
		
		$begin = "KEY-----";
		$end   = "-----END";
		
		$key = mb_substr($key, mb_strpos($key, $begin) + 8);
		$result = mb_substr($key, 0, mb_strpos($key, $end));
		
		return $result;
	}

	function convert_der_pem($key, $type = 'PUBLIC') {		
		$key = self::base64_set_padding(strtr($key, '-_', '+/'));		
		$result = "-----BEGIN ".$type." KEY-----\r\n".chunk_split($key, 64)."-----END ".$type." KEY-----";
		
		return $result;
	}
}
